"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const manifest_file_1 = require("@anycli/manifest-file");
class PluginCache extends manifest_file_1.default {
    constructor(file, cacheKey, name) {
        super(['@anycli/load', name].join(':'), file);
        this.file = file;
        this.cacheKey = cacheKey;
        this.name = name;
        this.type = 'cache';
        this.debug('file: %s cacheKey: %s', this.file, this.cacheKey);
    }
    async fetch(key, fn) {
        await this.lock.add('read');
        try {
            let [output, cacheKey] = await this.get(key, 'cache_key');
            if (cacheKey && cacheKey !== this.cacheKey) {
                await this.reset();
                output = undefined;
            }
            if (output)
                return output;
            this.debug('fetching', key);
            let input = await fn();
            try {
                await this.lock.add('write', { timeout: 200, reason: 'cache' });
                await this.set(['cache_key', this.cacheKey], [key, input]);
                return input;
            }
            catch (err) {
                this.debug(err);
                return input;
            }
            finally {
                await this.lock.remove('write');
            }
        }
        finally {
            await this.lock.remove('read');
        }
    }
}
exports.default = PluginCache;
