"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@anycli/command");
const config_1 = require("@anycli/config");
const cli_ux_1 = require("cli-ux");
const engine_1 = require("./engine");
exports.Engine = engine_1.default;
class Command extends command_1.default {
    async run() {
        cli_ux_1.default.config.errlog = this.config.errlog;
        const id = this.argv[0];
        await this.engine.runHook('init', { id });
        const cachedCommand = this.engine.findCommand(id);
        if (!cachedCommand)
            return this.commandNotFound(id);
        this.debug('found command', cachedCommand.id);
        const command = await cachedCommand.load();
        this.debug('loaded command', command.id);
        await command.run(this.argv.slice(1), { config: this.config });
    }
    async init(argv, opts) {
        const root = opts.root || module.parent.filename;
        this.argv = argv;
        this.config = await config_1.read({ root });
        this.initDebug(this.config);
        this.engine = new engine_1.default();
        this.config.engine = this.engine;
        await this.engine.load(this.config);
    }
    async commandNotFound(id) {
        await this.engine.runHook('command_not_found', { id });
        throw new Error(`command not found: ${id}`);
    }
}
function run(argv = process.argv.slice(2), opts = {}) {
    return Command.run(argv, opts);
}
exports.run = run;
exports.default = run;
